
require 'minxing_ruby_sdk/connection'
require 'minxing_ruby_sdk/account'
require 'minxing_ruby_sdk/util/util'
module MinxingRubySdk
  module Ocu
    TEXT = 0
    ARTICLE = 1

    class OcuAccount < MinxingRubySdk::Account
      
      attr_accessor :root_url, :ocuId, :ocuSecret, :api_prefix, :api_ocu_messages, :api_ocu_resources, :api_user_info

      def initialize(args={})
        config = YAML.load_file(MinxingRubySdk.root_path + '/settings.yml')
        
        defaults = {
          :api_prefix    => config['api']['api_prefix'],
          :api_ocu_messages    => config['api']['ocu_messages'],
          :api_ocu_resources => config['api']['ocu_resources'],
          :api_user_info => config['api']['user_info']
        }
        args.merge(defaults).each do |attr,val|
          instance_variable_set("@#{attr}", val) unless val.nil?
        end if args

        init_connection
      end

      def init_connection
        @connection = MinxingRubySdk::Connection.new @root_url
        @connection.tokenType = 'MAC'
        @connection.token = ''
      end

      def send_to_post (message, type, *userStr)
        headers = {}
        params = {}

        save_resource_and_get_id(message) if message.is_a? ArticleMessage

        params['body'] = message.body
        params['direct_to_user_ids'] = userStr[0] unless userStr.nil?
        params['content_type'] = type

        url = @root_url + @api_prefix + @api_ocu_messages

        beforeRequest(url, headers, params)
        post(url, true, headers, params)
      end

      def beforeRequest(url, headers, params)
        time = Time.new.to_i.to_s
        @connection.token = CGI::escape(@ocuId + ":" + Util.hmac_sha1(url + "?timestamp=" + time, @ocuSecret))
        headers["timestamp"] = time
      end

      def save_resource_and_get_id(articleMessage)
        url = @root_url + @api_prefix + @api_ocu_resources
        params = {}
        headers = {}
        articles = articleMessage.articles
        if articles.size > 0
          for i in 0...articles.size
            if articles[i].type == 'resource'
              resource = articles[i].resource[0]
              params['title'] = resource.title
              params['sub_title'] = resource.sub_title
              params['author'] = resource.author
              params['create_time'] = resource.create_time
              params['pic_url'] = resource.pic_url
              params['content'] = resource.description
              beforeRequest(url, headers, params)
              response_body = JSON post(url, true, headers, params)
              articles[i].resource_id = response_body['resource_id']
            end
          end
        end
      end

      def send_message_to_users(message, type, users)
        usersStr = users
        if users.is_a?(Array)
          usersStr = users.join(',')
        end
        send_to_post(message, type, usersStr)
      end
      def send_message_to_public(message, type)
        send_to_post(message, type)
      end
      def check_signature(request,params)
                
        if request.get?
          token = MinxingRubySdk::Util.url_param_decode params['token']
          timestamp = MinxingRubySdk::Util.url_param_decode params['timestamp']
          nonce = MinxingRubySdk::Util.url_param_decode params['nonce']
          open_id = MinxingRubySdk::Util.url_param_decode params['open_id']
        elsif request.post?
           token = request.headers['token']
           timestamp = request.headers['timestamp']
           nonce = request.headers['nonce']
           open_id = request.headers['open_id']
        end
        sign = MinxingRubySdk::Util.hmac_sha1(timestamp + nonce, @ocuSecret)

        open_id if CGI::escape(@ocuId + ":" +sign) == token
      end
      def get_user_info(open_id)
        headers = {}
        params = {}
        url = @root_url + @api_prefix + @api_user_info + "/" + open_id
        beforeRequest(url, headers, params)
        get(url,true,headers,params)
      end
    end
  end
end
